@@ -7,7 +7,7 @@
 processes in a portable way by using Python.
 """
 
-__version__ = "0.2.1"
+__version__ = "0.3.0"
 version_info = tuple([int(num) for num in __version__.split('.')])
 
 __all__ = [
@@ -23,8 +23,8 @@ __all__ = [
     "Process", "Popen",
     # functions
     "test", "pid_exists", "get_pid_list", "process_iter", "get_process_list",
-    "avail_phymem", "used_phymem", "total_virtmem", "avail_virtmem",
-    "used_virtmem", "cpu_times", "cpu_percent",
+    "phymem_usage", "virtmem_usage"
+    "cpu_times", "per_cpu_times", "cpu_percent", "per_cpu_percent"
     ]
 
 import sys
@@ -44,28 +44,48 @@ from psutil._compat import property
 from psutil._common import (STATUS_RUNNING, STATUS_IDLE, STATUS_SLEEPING,
                             STATUS_DISK_SLEEP, STATUS_STOPPED,
                             STATUS_TRACING_STOP, STATUS_ZOMBIE, STATUS_DEAD,
-                            STATUS_WAKING, STATUS_LOCKED)
+                            STATUS_WAKING, STATUS_LOCKED
+                            )
 
 # import the appropriate module for our platform only
 if sys.platform.lower().startswith("linux"):
-    from psutil._pslinux import *
-    __all__.extend(["cached_phymem", "phymem_buffers"])
+    import psutil._pslinux as _psplatform
+    from psutil._pslinux import (phymem_buffers,
+                                 cached_phymem,
+                                 IOPRIO_CLASS_NONE,
+                                 IOPRIO_CLASS_RT,
+                                 IOPRIO_CLASS_BE,
+                                 IOPRIO_CLASS_IDLE)
+    phymem_buffers = _psplatform.phymem_buffers
+    cached_phymem = _psplatform.cached_phymem
 
 elif sys.platform.lower().startswith("win32"):
-    from psutil._psmswindows import *
-    __all__.extend(["ABOVE_NORMAL_PRIORITY_CLASS", "BELOW_NORMAL_PRIORITY_CLASS",
-                    "HIGH_PRIORITY_CLASS", "IDLE_PRIORITY_CLASS",
-                    "NORMAL_PRIORITY_CLASS", "REALTIME_PRIORITY_CLASS"])
+    import psutil._psmswindows as _psplatform
+    from psutil._psmswindows import (ABOVE_NORMAL_PRIORITY_CLASS,
+                                     BELOW_NORMAL_PRIORITY_CLASS,
+                                     HIGH_PRIORITY_CLASS,
+                                     IDLE_PRIORITY_CLASS,
+                                     NORMAL_PRIORITY_CLASS,
+                                     REALTIME_PRIORITY_CLASS)
 
 elif sys.platform.lower().startswith("darwin"):
-    from psutil._psosx import *
+    import psutil._psosx as _psplatform
 
 elif sys.platform.lower().startswith("freebsd"):
-    from psutil._psbsd import *
+    import psutil._psbsd as _psplatform
 
 else:
     raise NotImplementedError('platform %s is not supported' % sys.platform)
 
+__all__.extend(_psplatform.__extra__all__)
+
+NUM_CPUS = _psplatform.NUM_CPUS
+BOOT_TIME = _psplatform.BOOT_TIME
+TOTAL_PHYMEM = _psplatform.phymem_usage()[0]
+
+get_pid_list = _psplatform.get_pid_list
+pid_exists = _psplatform.pid_exists
+
 
 class Process(object):
     """Represents an OS process."""
@@ -73,15 +93,16 @@ class Process(object):
     def __init__(self, pid):
         """Create a new Process object, raises NoSuchProcess if the PID
         does not exist, and ValueError if the parameter is not an
-        integer PID."""
+        integer PID.
+        """
         if not isinstance(pid, int):
-            raise ValueError("An integer is required")
+            raise ValueError("an integer is required")
         if not pid_exists(pid):
-            raise NoSuchProcess(pid, None, "no process found with PID %s" % pid)
+            raise NoSuchProcess(pid, None, "no process found with pid %s" % pid)
         self._pid = pid
-        # platform-specific modules define an PlatformProcess
+        # platform-specific modules define an _psplatform.Process
         # implementation class
-        self._platform_impl = PlatformProcess(pid)
+        self._platform_impl = _psplatform.Process(pid)
         self._last_sys_cpu_times = None
         self._last_proc_cpu_times = None
 
@@ -89,16 +110,12 @@ class Process(object):
         try:
             pid = self.pid
             name = repr(self.name)
-            cmdline = self.cmdline and repr(' '.join(self.cmdline))
         except NoSuchProcess:
             details = "(pid=%s (terminated))" % self.pid
         except AccessDenied:
             details = "(pid=%s)" % (self.pid)
         else:
-            if cmdline:
-                details = "(pid=%s, name=%s, cmdline=%s)" % (pid, name, cmdline)
-            else:
-                details = "(pid=%s, name=%s)" % (pid, name)
+            details = "(pid=%s, name=%s)" % (pid, name)
         return "%s.%s%s" % (self.__class__.__module__,
                             self.__class__.__name__, details)
 
@@ -172,12 +189,6 @@ class Process(object):
             raise AccessDenied(self.pid, self._platform_impl._process_name)
         return exe
 
-    @property
-    def path(self):
-        msg = "'path' property is deprecated; use 'os.path.dirname(exe)' instead"
-        warnings.warn(msg, DeprecationWarning)
-        return os.path.dirname(self.exe)
-
     @property
     def cmdline(self):
         """The command line process has been called with."""
@@ -214,23 +225,12 @@ class Process(object):
             """
             return self._platform_impl.get_process_gids()
 
-    @property
-    def uid(self):
-        """The real user id of the current process (deprecated)."""
-        warnings.warn("'uid' property is deprecated; use 'uids.real' instead",
-                      DeprecationWarning)
-        if os.name != 'posix':
-            return -1
-        return self.uids.real
-
-    @property
-    def gid(self):
-        """The real group id of the current process (deprecated)."""
-        warnings.warn("'gid' property is deprecated; use 'uids.real' instead",
-                      DeprecationWarning)
-        if os.name != 'posix':
-            return -1
-        return self.gids.real
+        @property
+        def terminal(self):
+            """The terminal associated with this process, if any,
+            else None.
+            """
+            return self._platform_impl.get_process_terminal()
 
     @property
     def username(self):
@@ -253,7 +253,7 @@ class Process(object):
         return self._platform_impl.get_process_create_time()
 
     # available for Windows and Linux only
-    if hasattr(PlatformProcess, "get_process_cwd"):
+    if hasattr(_psplatform.Process, "get_process_cwd"):
 
         def getcwd(self):
             """Return a string representing the process current working
@@ -262,7 +262,7 @@ class Process(object):
             return self._platform_impl.get_process_cwd()
 
     # Linux, BSD and Windows only
-    if hasattr(PlatformProcess, "get_process_io_counters"):
+    if hasattr(_psplatform.Process, "get_process_io_counters"):
 
         def get_io_counters(self):
             """Return process I/O statistics as a namedtuple including
@@ -272,7 +272,7 @@ class Process(object):
             return self._platform_impl.get_process_io_counters()
 
     # available only on Linux
-    if hasattr(PlatformProcess, "get_process_ionice"):
+    if hasattr(_psplatform.Process, "get_process_ionice"):
 
         def get_ionice(self):
             """Return process I/O niceness (priority) as a namedtuple."""
@@ -536,7 +536,6 @@ class Popen(Process):
                 raise AttributeError("%s instance has no attribute '%s'"
                                       %(self.__class__.__name__, name))
 
-
 def process_iter():
     """Return an iterator yielding a Process class instances for all
     running processes on the local machine.
@@ -554,8 +553,8 @@ def get_process_list():
     """
     return list(process_iter())
 
-def cpu_times():
-    """Return system CPU times as a namedtuple object.
+def cpu_times(percpu=False):
+    """Return system-wide CPU times as a namedtuple object.
     Every CPU time represents the time CPU has spent in the given mode.
     The attributes availability varies depending on the platform.
     Here follows a list of all available attributes:
@@ -566,13 +565,22 @@ def cpu_times():
      - iowait (Linux)
      - irq (Linux, FreeBSD)
      - softirq (Linux)
+
+    When percpu is True return a list of nameduples for each CPU.
+    First element of the list refers to first CPU, second element
+    to second CPU and so on.
+    The order of the list is consistent across calls.
     """
-    return get_system_cpu_times()
+    if not percpu:
+        return _psplatform.get_system_cpu_times()
+    else:
+        return _psplatform.get_system_per_cpu_times()
 
 
 _last_cpu_times = cpu_times()
+_last_per_cpu_times = cpu_times(percpu=True)
 
-def cpu_percent(interval=0.1):
+def cpu_percent(interval=0.1, percpu=False):
     """Return a float representing the current system-wide CPU
     utilization as a percentage.
 
@@ -583,33 +591,121 @@ def cpu_percent(interval=0.1):
     since last call or module import, returning immediately.
     In this case is recommended for accuracy that this function be
     called with at least 0.1 seconds between calls.
+
+    When percpu is True returns a list of floats representing the
+    utilization as a percentage for each CPU.
+    First element of the list refers to first CPU, second element
+    to second CPU and so on.
+    The order of the list is consistent across calls.
     """
     global _last_cpu_times
-
+    global _last_per_cpu_times
     blocking = interval is not None and interval > 0.0
-    if blocking:
-        t1 = cpu_times()
-        time.sleep(interval)
+
+    def calculate(t1, t2):
+        t1_all = sum(t1)
+        t1_busy = t1_all - t1.idle
+
+        t2_all = sum(t2)
+        t2_busy = t2_all - t2.idle
+
+        # this usually indicates a float precision issue
+        if t2_busy <= t1_busy:
+            return 0.0
+
+        busy_delta = t2_busy - t1_busy
+        all_delta = t2_all - t1_all
+        busy_perc = (busy_delta / all_delta) * 100
+        return round(busy_perc, 1)
+
+    # system-wide usage
+    if not percpu:
+        if blocking:
+            t1 = cpu_times()
+            time.sleep(interval)
+        else:
+            t1 = _last_cpu_times
+        _last_cpu_times = cpu_times()
+        return calculate(t1, _last_cpu_times)
+    # per-cpu usage
     else:
-        t1 = _last_cpu_times
+        ret = []
+        if blocking:
+            tot1 = cpu_times(percpu=True)
+            time.sleep(interval)
+        else:
+            tot1 = _last_per_cpu_times
+        _last_per_cpu_times = cpu_times(percpu=True)
+        for t1, t2 in zip(tot1, _last_per_cpu_times):
+            ret.append(calculate(t1, t2))
+        return ret
 
-    t1_all = sum(t1)
-    t1_busy = t1_all - t1.idle
+def phymem_usage():
+    """Return the amount of total, used and free physical memory
+    on the system in bytes plus the percentage usage.
+    """
+    return _psplatform.phymem_usage()
 
-    t2 = cpu_times()
-    t2_all = sum(t2)
-    t2_busy = t2_all - t2.idle
+def virtmem_usage():
+    """Return the amount of total, used and free virtual memory
+    on the system in bytes plus the percentage usage.
 
-    _last_cpu_times = t1
-    # this usually indicates a float precision issue
-    if t2_busy <= t1_busy:
-        return 0.0
+    On Linux they match the values returned by free command line utility.
+    On OS X and FreeBSD they represent the same values as returned by
+    sysctl vm.vmtotal. On Windows they are determined by reading the
+    PageFile values of MEMORYSTATUSEX structure.
+    """
+    return _psplatform.virtmem_usage()
 
-    busy_delta = t2_busy - t1_busy
-    all_delta = t2_all - t1_all
-    busy_perc = (busy_delta / all_delta) * 100
-    return round(busy_perc, 1)
+def disk_usage(path):
+    """Return disk usage statistics about the given path as a namedtuple
+    including total, used and free space expressed in bytes plus the
+    percentage usage.
+    """
+    return _psplatform.get_disk_usage(path)
 
+def disk_partitions(all=False):
+    """Return mounted partitions as a list of namedtuples including
+    device, mount point and filesystem type.
+
+    If "all" parameter is False return physical devices only and ignore
+    all others.
+    """
+    return _psplatform.disk_partitions(all)
+
+
+def _deprecated(replacement):
+    # a decorator which can be used to mark functions as deprecated
+    def outer(fun):
+        def inner(*args, **kwargs):
+            msg = "psutil.%s is deprecated; use %s instead" \
+                  % (fun.__name__, replacement)
+            warnings.warn(msg, category=DeprecationWarning, stacklevel=2)
+            return fun(*args, **kwargs)
+        return inner
+    return outer
+
+# --- deprecated functions
+
+@_deprecated("psutil.phymem_usage")
+def avail_phymem():
+    return phymem_usage().free
+
+@_deprecated("psutil.phymem_usage")
+def used_phymem():
+    return phymem_usage().used
+
+@_deprecated("psutil.virtmem_usage")
+def total_virtmem():
+    return virtmem_usage().total
+
+@_deprecated("psutil.virtmem_usage")
+def used_virtmem():
+    return virtmem_usage().used
+
+@_deprecated("psutil.virtmem_usage")
+def avail_virtmem():
+    return virtmem_usage().free
 
 def test():
     """List info of all currently running processes emulating a
