@@ -14,57 +14,51 @@ import _psutil_posix
 import _psutil_linux
 from psutil import _psposix
 from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
-from psutil._compat import namedtuple
 from psutil._common import *
 
-# Linux specific extended namespace
-__all__ = base_module_namespace[:]
-__all__.extend([
-    "IOPRIO_CLASS_NONE", "IOPRIO_CLASS_RT", "IOPRIO_CLASS_BE", "IOPRIO_CLASS_IDLE",
-    "phymem_buffers", "cached_phymem",
-    ])
+__extra__all__ = [
+    "IOPRIO_CLASS_NONE", "IOPRIO_CLASS_RT", "IOPRIO_CLASS_BE",
+    "IOPRIO_CLASS_IDLE",
+    "phymem_buffers", "cached_phymem"]
 
 
 def _get_boot_time():
     """Return system boot time (epoch in seconds)"""
     f = open('/proc/stat', 'r')
-    for line in f:
-        if line.startswith('btime'):
-            f.close()
-            return float(line.strip().split()[1])
+    try:
+        for line in f:
+            if line.startswith('btime'):
+                return float(line.strip().split()[1])
+        raise RuntimeError("line not found")
+    finally:
+        f.close()
 
 def _get_num_cpus():
     """Return the number of CPUs on the system"""
     num = 0
     f = open('/proc/cpuinfo', 'r')
-    for line in f:
-        if line.startswith('processor'):
-            num += 1
-    f.close()
-    return num
-
-def _get_total_phymem():
-    """Return the total amount of physical memory, in bytes"""
-    f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('MemTotal:'):
-            f.close()
-            return int(line.split()[1]) * 1024
+    try:
+        for line in f:
+            if line.startswith('processor'):
+                num += 1
+        if num == 0:
+            raise RuntimeError("line not found")
+        return num
+    finally:
+        f.close()
 
 
 # Number of clock ticks per second
 _CLOCK_TICKS = os.sysconf(os.sysconf_names["SC_CLK_TCK"])
+_TERMINAL_MAP = _psposix._get_terminal_map()
 BOOT_TIME = _get_boot_time()
 NUM_CPUS = _get_num_cpus()
-TOTAL_PHYMEM = _get_total_phymem()
 # ioprio_* constants http://linux.die.net/man/2/ioprio_get
 IOPRIO_CLASS_NONE = 0
 IOPRIO_CLASS_RT = 1
 IOPRIO_CLASS_BE = 2
 IOPRIO_CLASS_IDLE = 3
 
-del _get_boot_time, _get_num_cpus, _get_total_phymem
-
 # http://students.mimuw.edu.pl/lxr/source/include/net/tcp_states.h
 _TCP_STATES_TABLE = {"01" : "ESTABLISHED",
                      "02" : "SYN_SENT",
@@ -79,55 +73,20 @@ _TCP_STATES_TABLE = {"01" : "ESTABLISHED",
                      "0B" : "CLOSING"
                      }
 
-def avail_phymem():
-    """Return the amount of physical memory available, in bytes."""
-    f = open('/proc/meminfo', 'r')
-    free = None
-    _flag = False
-    for line in f:
-        if line.startswith('MemFree:'):
-            free = int(line.split()[1]) * 1024
-            break
-    f.close()
-    return free
-
-def used_phymem():
-    """"Return the amount of physical memory used, in bytes."""
-    return (TOTAL_PHYMEM - avail_phymem())
-
-def total_virtmem():
-    """"Return the total amount of virtual memory, in bytes."""
-    f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('SwapTotal:'):
-            f.close()
-            return int(line.split()[1]) * 1024
-
-def avail_virtmem():
-    """Return the amount of virtual memory currently in use on the
-    system, in bytes.
-    """
-    f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('SwapFree:'):
-            f.close()
-            return int(line.split()[1]) * 1024
-
-def used_virtmem():
-    """Return the amount of used memory currently in use on the system,
-    in bytes.
-    """
-    return total_virtmem() - avail_virtmem()
+# --- system memory functions
 
 def cached_phymem():
     """Return the amount of cached memory on the system, in bytes.
     This reflects the "cached" column of free command line utility.
     """
     f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('Cached:'):
-            f.close()
-            return int(line.split()[1]) * 1024
+    try:
+        for line in f:
+            if line.startswith('Cached:'):
+                return int(line.split()[1]) * 1024
+        raise RuntimeError("line not found")
+    finally:
+        f.close()
 
 def phymem_buffers():
     """Return the amount of physical memory buffers used by the
@@ -135,23 +94,123 @@ def phymem_buffers():
     This reflects the "buffers" column of free command line utility.
     """
     f = open('/proc/meminfo', 'r')
-    for line in f:
-        if line.startswith('Buffers:'):
-            f.close()
-            return int(line.split()[1]) * 1024
+    try:
+        for line in f:
+            if line.startswith('Buffers:'):
+                return int(line.split()[1]) * 1024
+        raise RuntimeError("line not found")
+    finally:
+        f.close()
+
+def phymem_usage():
+    # total, used and free values are matched against free cmdline utility
+    # the percentage matches top/htop and gnome-system-monitor
+    f = open('/proc/meminfo', 'r')
+    try:
+        total = free = buffers = cached = None
+        for line in f:
+            if line.startswith('MemTotal:'):
+                total = int(line.split()[1]) * 1024
+            elif line.startswith('MemFree:'):
+                free = int(line.split()[1]) * 1024
+            elif line.startswith('Buffers:'):
+                buffers = int(line.split()[1]) * 1024
+            elif line.startswith('Cached:'):
+                cached = int(line.split()[1]) * 1024
+                break
+        used = total - free
+        percent = usage_percent(total - (free + buffers + cached), total,
+                                _round=1)
+        return ntuple_sysmeminfo(total, used, free, percent)
+    finally:
+        f.close()
+
+
+def virtmem_usage():
+    f = open('/proc/meminfo', 'r')
+    try:
+        total = free = None
+        for line in f:
+            if line.startswith('SwapTotal:'):
+                total = int(line.split()[1]) * 1024
+            elif line.startswith('SwapFree:'):
+                free = int(line.split()[1]) * 1024
+            if total is not None and free is not None:
+                break
+        assert total is not None and free is not None
+        used = total - free
+        percent = usage_percent(used, total, _round=1)
+        return ntuple_sysmeminfo(total, used, free, percent)
+    finally:
+        f.close()
+
+
+# --- system CPU functions
 
 def get_system_cpu_times():
-    """Return a dict representing the following CPU times:
+    """Return a named tuple representing the following CPU times:
     user, nice, system, idle, iowait, irq, softirq.
     """
     f = open('/proc/stat', 'r')
-    values = f.readline().split()
-    f.close()
+    try:
+        values = f.readline().split()
+    finally:
+        f.close()
 
     values = values[1:8]
     values = tuple([float(x) / _CLOCK_TICKS for x in values])
     return ntuple_sys_cputimes(*values[:7])
 
+def get_system_per_cpu_times():
+    """Return a list of namedtuple representing the CPU times
+    for every CPU available on the system.
+    """
+    cpus = []
+    f = open('/proc/stat', 'r')
+    # get rid of the first line who refers to system wide CPU stats
+    try:
+        f.readline()
+        for line in f.readlines():
+            if line.startswith('cpu'):
+                values = line.split()[1:8]
+                values = tuple([float(x) / _CLOCK_TICKS for x in values])
+                entry = ntuple_sys_cputimes(*values[:7])
+                cpus.append(entry)
+        return cpus
+    finally:
+        f.close()
+
+
+# --- system disk functions
+
+def disk_partitions(all=False):
+    """Return mounted disk partitions as a list of nameduples"""
+    phydevs = []
+    f = open("/proc/filesystems", "r")
+    try:
+        for line in f:
+            if not line.startswith("nodev"):
+                phydevs.append(line.strip())
+    finally:
+        f.close()
+
+    retlist = []
+    partitions = _psutil_linux.get_disk_partitions()
+    for partition in partitions:
+        device, mountpoint, fstype = partition
+        if device == 'none':
+            device = ''
+        if not all:
+            if device == '' or fstype not in phydevs:
+                continue
+        ntuple = ntuple_partition(device, mountpoint, fstype)
+        retlist.append(ntuple)
+    return retlist
+
+get_disk_usage = _psposix.get_disk_usage
+
+# --- process functions
+
 def get_pid_list():
     """Returns a list of PIDs currently running on the system."""
     pids = [int(x) for x in os.listdir('/proc') if x.isdigit()]
@@ -163,7 +222,6 @@ def pid_exists(pid):
     """Check For the existence of a unix pid."""
     return _psposix.pid_exists(pid)
 
-
 # taken from /fs/proc/array.c
 _status_map = {"R" : STATUS_RUNNING,
                "S" : STATUS_SLEEPING,
@@ -197,7 +255,7 @@ def wrap_exceptions(callable):
     return wrapper
 
 
-class LinuxProcess(object):
+class Process(object):
     """Linux process implementation."""
 
     __slots__ = ["pid", "_process_name"]
@@ -261,22 +319,39 @@ class LinuxProcess(object):
         finally:
             f.close()
 
+    @wrap_exceptions
+    def get_process_terminal(self):
+        if self.pid == 0:
+            return None   # special case for kernel process
+        f = open("/proc/%s/stat" % self.pid)
+        try:
+            tty_nr = int(f.read().split(' ')[6])
+        finally:
+            f.close()
+        try:
+            return _TERMINAL_MAP[tty_nr]
+        except KeyError:
+            return None
+
     @wrap_exceptions
     def get_process_io_counters(self):
         # special case for 0 (kernel process) PID
         if self.pid == 0:
             return ntuple_io(0, 0, 0, 0)
         f = open("/proc/%s/io" % self.pid)
-        for line in f:
-            if line.startswith("rchar"):
-                read_count = int(line.split()[1])
-            elif line.startswith("wchar"):
-                write_count = int(line.split()[1])
-            elif line.startswith("read_bytes"):
-                read_bytes = int(line.split()[1])
-            elif line.startswith("write_bytes"):
-                write_bytes = int(line.split()[1])
-        return ntuple_io(read_count, write_count, read_bytes, write_bytes)
+        try:
+            for line in f:
+                if line.startswith("rchar"):
+                    read_count = int(line.split()[1])
+                elif line.startswith("wchar"):
+                    write_count = int(line.split()[1])
+                elif line.startswith("read_bytes"):
+                    read_bytes = int(line.split()[1])
+                elif line.startswith("write_bytes"):
+                    write_bytes = int(line.split()[1])
+            return ntuple_io(read_count, write_count, read_bytes, write_bytes)
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_cpu_times(self):
@@ -284,8 +359,10 @@ class LinuxProcess(object):
         if self.pid == 0:
             return ntuple_cputimes(0.0, 0.0)
         f = open("/proc/%s/stat" % self.pid)
-        st = f.read().strip()
-        f.close()
+        try:
+            st = f.read().strip()
+        finally:
+            f.close()
         # ignore the first two values ("pid (exe)")
         st = st[st.find(')') + 2:]
         values = st.split(' ')
@@ -306,8 +383,10 @@ class LinuxProcess(object):
         if self.pid == 0:
             return BOOT_TIME
         f = open("/proc/%s/stat" % self.pid)
-        st = f.read().strip()
-        f.close()
+        try:
+            st = f.read().strip()
+        finally:
+            f.close()
         # ignore the first two values ("pid (exe)")
         st = st[st.find(')') + 2:]
         values = st.split(' ')
@@ -324,18 +403,20 @@ class LinuxProcess(object):
         if self.pid == 0:
             return ntuple_meminfo(0, 0)
         f = open("/proc/%s/status" % self.pid)
-        virtual_size = 0
-        resident_size = 0
-        _flag = False
-        for line in f:
-            if (not _flag) and line.startswith("VmSize:"):
-                virtual_size = int(line.split()[1]) * 1024
-                _flag = True
-            elif line.startswith("VmRSS"):
-                resident_size = int(line.split()[1]) * 1024
-                break
-        f.close()
-        return ntuple_meminfo(resident_size, virtual_size)
+        try:
+            virtual_size = 0
+            resident_size = 0
+            _flag = False
+            for line in f:
+                if (not _flag) and line.startswith("VmSize:"):
+                    virtual_size = int(line.split()[1]) * 1024
+                    _flag = True
+                elif line.startswith("VmRSS"):
+                    resident_size = int(line.split()[1]) * 1024
+                    break
+            return ntuple_meminfo(resident_size, virtual_size)
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_process_cwd(self):
@@ -352,10 +433,13 @@ class LinuxProcess(object):
         if self.pid == 0:
             return 0
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith("Threads:"):
-                f.close()
-                return int(line.split()[1])
+        try:
+            for line in f:
+                if line.startswith("Threads:"):
+                    return int(line.split()[1])
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_process_threads(self):
@@ -373,8 +457,10 @@ class LinuxProcess(object):
                     # disappeared on us
                     continue
                 raise
-            st = f.read().strip()
-            f.close()
+            try:
+                st = f.read().strip()
+            finally:
+                f.close()
             # ignore the first two values ("pid (exe)")
             st = st[st.find(')') + 2:]
             values = st.split(' ')
@@ -387,9 +473,11 @@ class LinuxProcess(object):
     @wrap_exceptions
     def get_process_nice(self):
         #f = open('/proc/%s/stat' % self.pid, 'r')
-        #data = f.read()
-        #f.close()
-        #return int(data.split()[18])
+        #try:
+        #   data = f.read()
+        #   return int(data.split()[18])
+        #finally:
+        #   f.close()
 
         # Use C implementation
         return _psutil_posix.getpriority(self.pid)
@@ -431,16 +519,20 @@ class LinuxProcess(object):
         if self.pid == 0:
             return 0
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith("State:"):
-                f.close()
-                letter = line.split()[1]
-                if letter in _status_map:
-                    return _status_map[letter]
-                return constant(-1, '?')
+        try:
+            for line in f:
+                if line.startswith("State:"):
+                    letter = line.split()[1]
+                    if letter in _status_map:
+                        return _status_map[letter]
+                    return constant(-1, '?')
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_open_files(self):
+        if self.pid == 0:
+            return []
         retlist = []
         files = os.listdir("/proc/%s/fd" % self.pid)
         for fd in files:
@@ -483,22 +575,25 @@ class LinuxProcess(object):
         def process(file, family, _type):
             retlist = []
             f = open(file)
-            f.readline()  # skip the first line
-            for line in f:
-                _, laddr, raddr, status, _, _, _, _, _, inode = line.split()[:10]
-                if inode in inodes:
-                    laddr = self._decode_address(laddr, family)
-                    raddr = self._decode_address(raddr, family)
-                    if _type == socket.SOCK_STREAM:
-                        status = _TCP_STATES_TABLE[status]
-                    else:
-                        status = ""
-                    fd = int(inodes[inode])
-                    conn = ntuple_connection(fd, family, _type, laddr,
-                                             raddr, status)
-                    retlist.append(conn)
-            f.close()
-            return retlist
+            try:
+                f.readline()  # skip the first line
+                for line in f:
+                    _, laddr, raddr, status, _, _, _, _, _, inode = \
+                                                            line.split()[:10]
+                    if inode in inodes:
+                        laddr = self._decode_address(laddr, family)
+                        raddr = self._decode_address(raddr, family)
+                        if _type == socket.SOCK_STREAM:
+                            status = _TCP_STATES_TABLE[status]
+                        else:
+                            status = ""
+                        fd = int(inodes[inode])
+                        conn = ntuple_connection(fd, family, _type, laddr,
+                                                 raddr, status)
+                        retlist.append(conn)
+                return retlist
+            finally:
+                f.close()
 
         tcp4 = process("/proc/net/tcp", socket.AF_INET, socket.SOCK_STREAM)
         tcp6 = process("/proc/net/tcp6", socket.AF_INET6, socket.SOCK_STREAM)
@@ -517,33 +612,42 @@ class LinuxProcess(object):
         if self.pid == 0:
             return 0
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith("PPid:"):
-                # PPid: nnnn
-                f.close()
-                return int(line.split()[1])
+        try:
+            for line in f:
+                if line.startswith("PPid:"):
+                    # PPid: nnnn
+                    return int(line.split()[1])
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_process_uids(self):
         if self.pid == 0:
             return ntuple_uids(0, 0, 0)
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith('Uid:'):
-                f.close()
-                _, real, effective, saved, fs = line.split()
-                return ntuple_uids(int(real), int(effective), int(saved))
+        try:
+            for line in f:
+                if line.startswith('Uid:'):
+                    _, real, effective, saved, fs = line.split()
+                    return ntuple_uids(int(real), int(effective), int(saved))
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
 
     @wrap_exceptions
     def get_process_gids(self):
         if self.pid == 0:
             return ntuple_uids(0, 0, 0)
         f = open("/proc/%s/status" % self.pid)
-        for line in f:
-            if line.startswith('Gid:'):
-                f.close()
-                _, real, effective, saved, fs = line.split()
-                return ntuple_gids(int(real), int(effective), int(saved))
+        try:
+            for line in f:
+                if line.startswith('Gid:'):
+                    _, real, effective, saved, fs = line.split()
+                    return ntuple_gids(int(real), int(effective), int(saved))
+            raise RuntimeError("line not found")
+        finally:
+            f.close()
 
     @staticmethod
     def _decode_address(addr, family):
@@ -581,6 +685,3 @@ class LinuxProcess(object):
             ip = socket.inet_ntop(socket.AF_INET6,
                                 struct.pack('>4I', *struct.unpack('<4I', ip)))
         return (ip, port)
-
-PlatformProcess = LinuxProcess
-
