@@ -5,9 +5,6 @@
 
 import errno
 import os
-import subprocess
-import socket
-import re
 import sys
 import platform
 
@@ -17,20 +14,18 @@ from psutil._compat import namedtuple
 from psutil._common import *
 
 # Windows specific extended namespace
-__all__ = base_module_namespace[:]
-__all__.extend(["ABOVE_NORMAL_PRIORITY_CLASS", "BELOW_NORMAL_PRIORITY_CLASS",
-                "HIGH_PRIORITY_CLASS", "IDLE_PRIORITY_CLASS",
-                "NORMAL_PRIORITY_CLASS", "REALTIME_PRIORITY_CLASS"
-               ])
+__extra__all__ = ["ABOVE_NORMAL_PRIORITY_CLASS", "BELOW_NORMAL_PRIORITY_CLASS",
+                  "HIGH_PRIORITY_CLASS", "IDLE_PRIORITY_CLASS",
+                  "NORMAL_PRIORITY_CLASS", "REALTIME_PRIORITY_CLASS"]
 
 
 # --- module level constants (gets pushed up to psutil module)
 
 NUM_CPUS = _psutil_mswindows.get_num_cpus()
-TOTAL_PHYMEM = _psutil_mswindows.get_total_phymem()
 BOOT_TIME = _psutil_mswindows.get_system_uptime()
 _WIN2000 = platform.win32_ver()[0] == '2000'
 ERROR_ACCESS_DENIED = 5
+WAIT_TIMEOUT = 0x00000102 # 258 in decimal
 
 # process priority constants:
 # http://msdn.microsoft.com/en-us/library/ms686219(v=vs.85).aspx
@@ -41,42 +36,79 @@ from _psutil_mswindows import (ABOVE_NORMAL_PRIORITY_CLASS,
                                NORMAL_PRIORITY_CLASS,
                                REALTIME_PRIORITY_CLASS)
 
-
 # --- public functions
 
-def avail_phymem():
-    "Return the amount of physical memory available on the system, in bytes."
-    return _psutil_mswindows.get_avail_phymem()
+def phymem_usage():
+    """Physical system memory as a (total, used, free) tuple."""
+    all = _psutil_mswindows.get_system_phymem()
+    total, free, total_pagef, avail_pagef, total_virt, free_virt, percent = all
+    used = total - free
+    return ntuple_sysmeminfo(total, used, free, round(percent, 1))
 
-def used_phymem():
-    "Return the amount of physical memory currently in use on the system, in bytes."
-    return TOTAL_PHYMEM - _psutil_mswindows.get_avail_phymem()
+def virtmem_usage():
+    """Virtual system memory as a (total, used, free) tuple."""
+    all = _psutil_mswindows.get_system_phymem()
+    total_virt = all[4]
+    free_virt = all[5]
+    used = total_virt - free_virt
+    percent = usage_percent(used, total_virt, _round=1)
+    return ntuple_sysmeminfo(total_virt, used, free_virt, percent)
 
-def total_virtmem():
-    "Return the amount of total virtual memory available on the system, in bytes."
-    return _psutil_mswindows.get_total_virtmem()
+def get_disk_usage(path):
+    """Return disk usage associated with path."""
+    try:
+        total, free = _psutil_mswindows.get_disk_usage(path)
+    except WindowsError, err:
+        if not os.path.exists(path):
+            raise OSError(errno.ENOENT, "No such file or directory: '%s'" % path)
+        raise
+    used = total - free
+    percent = usage_percent(used, total, _round=1)
+    return ntuple_diskinfo(total, used, free, percent)
 
-def avail_virtmem():
-    "Return the amount of virtual memory currently in use on the system, in bytes."
-    return _psutil_mswindows.get_avail_virtmem()
+def disk_partitions(all):
+    """Return disk partitions."""
+    retlist = []
+    drive_letters = _psutil_mswindows.win32_GetLogicalDriveStrings()
+    for letter in drive_letters:
+        mountpoint = letter
+        type = _psutil_mswindows.win32_GetDriveType(letter)
+        if not os.path.exists(mountpoint):
+            # usually a CD-ROM device with no disk inserted
+            mountpoint = ""
+        if not all:
+            if type not in ('cdrom', 'fixed', 'removable'):
+                continue
+            if not mountpoint:
+                continue
+        ntuple = ntuple_partition(letter, mountpoint, type)
+        retlist.append(ntuple)
+    return retlist
 
-def used_virtmem():
-    """Return the amount of used memory currently in use on the system, in bytes."""
-    return _psutil_mswindows.get_total_virtmem() - _psutil_mswindows.get_avail_virtmem()
 
 _cputimes_ntuple = namedtuple('cputimes', 'user system idle')
+
 def get_system_cpu_times():
     """Return system CPU times as a named tuple."""
-    user, system, idle = _psutil_mswindows.get_system_cpu_times()
+    user, system, idle = 0, 0, 0
+    # computes system global times summing each processor value
+    for cpu_time in _psutil_mswindows.get_system_cpu_times():
+        user += cpu_time[0]
+        system += cpu_time[1]
+        idle += cpu_time[2]
     return _cputimes_ntuple(user, system, idle)
 
-def get_pid_list():
-    """Returns a list of PIDs currently running on the system."""
-    return _psutil_mswindows.get_pid_list()
-
-def pid_exists(pid):
-    return _psutil_mswindows.pid_exists(pid)
+def get_system_per_cpu_times():
+    """Return system per-CPU times as a list of named tuples."""
+    ret = []
+    for cpu_t in _psutil_mswindows.get_system_cpu_times():
+        user, system, idle = cpu_t
+        item = _cputimes_ntuple(user, system, idle)
+        ret.append(item)
+    return ret
 
+get_pid_list = _psutil_mswindows.get_pid_list
+pid_exists = _psutil_mswindows.pid_exists
 
 # --- decorator
 
@@ -97,7 +129,7 @@ def wrap_exceptions(callable):
     return wrapper
 
 
-class WindowsProcess(object):
+class Process(object):
     """Wrapper class around underlying C implementation."""
 
     __slots__ = ["pid", "_process_name"]
@@ -131,18 +163,6 @@ class WindowsProcess(object):
         """Return process parent pid."""
         return _psutil_mswindows.get_process_ppid(self.pid)
 
-    def get_process_uid(self):
-        # no such thing as uid on Windows
-        if not pid_exists(self.pid):
-            raise NoSuchProcess(self.pid, self._process_name)
-        return -1
-
-    def get_process_gid(self):
-        # no such thing as gid on Windows
-        if not pid_exists(self.pid):
-            raise NoSuchProcess(self.pid, self._process_name)
-        return -1
-
     @wrap_exceptions
     def get_memory_info(self):
         """Returns a tuple or RSS/VMS memory usage in bytes."""
@@ -165,7 +185,7 @@ class WindowsProcess(object):
             # WaitForSingleObject() expects time in milliseconds
             timeout = int(timeout * 1000)
         ret = _psutil_mswindows.process_wait(self.pid, timeout)
-        if ret == -1:
+        if ret == WAIT_TIMEOUT:
             raise TimeoutExpired(self.pid, self._process_name)
         return ret
 
@@ -233,7 +253,7 @@ class WindowsProcess(object):
                 file = file.decode('utf8')
             if file.startswith('\\Device\\'):
                 rawdrive = '\\'.join(file.split('\\')[:3])
-                driveletter = _psutil_mswindows._QueryDosDevice(rawdrive)
+                driveletter = _psutil_mswindows.win32_QueryDosDevice(rawdrive)
                 file = file.replace(rawdrive, driveletter)
                 if os.path.isfile(file) and file not in retlist:
                     ntuple = ntuple_openfile(file, -1)
@@ -265,6 +285,3 @@ class WindowsProcess(object):
             return STATUS_STOPPED
         else:
             return STATUS_RUNNING
-
-PlatformProcess = WindowsProcess
-
