@@ -14,9 +14,11 @@ import re
 import sys
 import warnings
 import time
+import glob
 
 from psutil.error import AccessDenied, NoSuchProcess, TimeoutExpired
 from psutil._compat import namedtuple
+from psutil._common import ntuple_diskinfo, usage_percent
 
 
 def pid_exists(pid):
@@ -86,6 +88,26 @@ def wait_pid(pid, timeout=None):
                 # should never happen
                 raise RuntimeError("unknown process exit status")
 
+def get_disk_usage(path):
+    """Return disk usage associated with path."""
+    st = os.statvfs(path)
+    free = (st.f_bavail * st.f_frsize)
+    total = (st.f_blocks * st.f_frsize)
+    used = (st.f_blocks - st.f_bfree) * st.f_frsize
+    percent = usage_percent(used, total, _round=1)
+    # NB: the percentage is -5% than what shown by df due to
+    # reserved blocks that we are currently not considering:
+    # http://goo.gl/sWGbH
+    return ntuple_diskinfo(total, used, free, percent)
+
+def _get_terminal_map():
+    ret = {}
+    ls = glob.glob('/dev/tty*') + glob.glob('/dev/pts/*')
+    for name in ls:
+        assert name not in ret
+        ret[os.stat(name).st_rdev] = name
+    return ret
+
 
 class LsofParser:
     """A wrapper for lsof command line utility.
