@@ -119,10 +119,12 @@ kill_process(PyObject* self, PyObject* args)
     HANDLE hProcess;
     long pid;
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
-    if (pid == 0)
+    }
+    if (pid == 0) {
         return AccessDenied();
+    }
 
     hProcess = OpenProcess(PROCESS_TERMINATE, FALSE, pid);
     if (hProcess == NULL) {
@@ -161,12 +163,15 @@ process_wait(PyObject* self, PyObject* args)
     long pid;
     long timeout;
 
-    if (! PyArg_ParseTuple(args, "ll", &pid, &timeout))
+    if (! PyArg_ParseTuple(args, "ll", &pid, &timeout)) {
         return NULL;
-    if (pid == 0)
+    }
+    if (pid == 0) {
         return AccessDenied();
-    if (timeout == 0)
+    }
+    if (timeout == 0) {
         timeout = INFINITE;
+    }
 
     hProcess = OpenProcess(SYNCHRONIZE | PROCESS_QUERY_INFORMATION, FALSE, pid);
     if (hProcess == NULL) {
@@ -193,7 +198,7 @@ process_wait(PyObject* self, PyObject* args)
     }
     if (retVal == WAIT_TIMEOUT) {
         CloseHandle(hProcess);
-        return Py_BuildValue("i", -1);
+        return Py_BuildValue("l", WAIT_TIMEOUT);
     }
 
     // get the exit code; note: subprocess module (erroneously?) uses
@@ -204,9 +209,9 @@ process_wait(PyObject* self, PyObject* args)
     }
     CloseHandle(hProcess);
 #if PY_MAJOR_VERSION >= 3
-    return PyLong_FromLong((int) ExitCode);
+    return PyLong_FromLong((long) ExitCode);
 #else
-    return PyInt_FromLong((int) ExitCode);
+    return PyInt_FromLong((long) ExitCode);
 #endif
 }
 
@@ -236,15 +241,16 @@ get_process_cpu_times(PyObject* self, PyObject* args)
     }
 
     if (! GetProcessTimes(hProcess, &ftCreate, &ftExit, &ftKernel, &ftUser)) {
+        CloseHandle(hProcess);
         if (GetLastError() == ERROR_ACCESS_DENIED) {
             // usually means the process has died so we throw a NoSuchProcess
             // here
-            CloseHandle(hProcess);
             return NoSuchProcess();
         }
-        PyErr_SetFromWindowsErr(0);
-        CloseHandle(hProcess);
-        return NULL;
+        else {
+            PyErr_SetFromWindowsErr(0);
+            return NULL;
+        }
     }
 
     CloseHandle(hProcess);
@@ -297,15 +303,15 @@ get_process_create_time(PyObject* self, PyObject* args)
     }
 
     if (! GetProcessTimes(hProcess, &ftCreate, &ftExit, &ftKernel, &ftUser)) {
+        CloseHandle(hProcess);
         if (GetLastError() == ERROR_ACCESS_DENIED) {
             // usually means the process has died so we throw a NoSuchProcess here
             return NoSuchProcess();
         }
         else {
             PyErr_SetFromWindowsErr(0);
+            return NULL;
         }
-        CloseHandle(hProcess);
-        return NULL;
     }
 
     CloseHandle(hProcess);
@@ -348,8 +354,9 @@ get_process_name(PyObject* self, PyObject* args) {
     int pid_return;
     PyObject* name;
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
+    }
 
     if (pid == 0) {
         return Py_BuildValue("s", "System Idle Process");
@@ -367,8 +374,9 @@ get_process_name(PyObject* self, PyObject* args) {
     }
 
     name = get_name(pid);
-    if (name == NULL)
+    if (name == NULL) {
         return NULL;  // exception set in get_name()
+    }
     return name;
 }
 
@@ -382,10 +390,12 @@ get_process_ppid(PyObject* self, PyObject* args) {
     int pid_return;
     PyObject* ppid;
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
-    if ((pid == 0) || (pid == 4))
+    }
+    if ((pid == 0) || (pid == 4)) {
         return Py_BuildValue("l", 0);
+    }
 
     pid_return = pid_is_running(pid);
     if (pid_return == 0) {
@@ -396,8 +406,9 @@ get_process_ppid(PyObject* self, PyObject* args) {
     }
 
     ppid = get_ppid(pid);
-    if (ppid == NULL)
+    if (ppid == NULL) {
         return NULL;  // exception set in get_ppid()
+    }
     return ppid;
 }
 
@@ -410,10 +421,12 @@ get_process_cmdline(PyObject* self, PyObject* args) {
     int pid_return;
     PyObject* arglist;
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
-    if ((pid == 0) || (pid == 4))
+    }
+    if ((pid == 0) || (pid == 4)) {
         return Py_BuildValue("[]");
+    }
 
     pid_return = pid_is_running(pid);
     if (pid_return == 0) {
@@ -456,9 +469,11 @@ get_memory_info(PyObject* self, PyObject* args)
     }
 
     if (! GetProcessMemoryInfo(hProcess, &counters, sizeof(counters)) ) {
+        CloseHandle(hProcess);
         return PyErr_SetFromWindowsErr(0);
     }
 
+    CloseHandle(hProcess);
     return Py_BuildValue("(nn)", counters.WorkingSetSize, counters.PagefileUsage);
 }
 
@@ -468,7 +483,7 @@ get_memory_info(PyObject* self, PyObject* args)
  * in bytes.
  */
 static PyObject*
-get_total_phymem(PyObject* self, PyObject* args)
+get_system_phymem(PyObject* self, PyObject* args)
 {
     MEMORYSTATUSEX memInfo;
     memInfo.dwLength = sizeof(MEMORYSTATUSEX);
@@ -477,58 +492,15 @@ get_total_phymem(PyObject* self, PyObject* args)
         return PyErr_SetFromWindowsErr(0);
     }
 
-    return Py_BuildValue("L", memInfo.ullTotalPhys);
-}
-
-
-/*
- * Return a Python integer indicating the total amount of virtual memory
- * in bytes.
- */
-static PyObject*
-get_total_virtmem(PyObject* self, PyObject* args)
-{
-    MEMORYSTATUSEX memInfo;
-    memInfo.dwLength = sizeof(MEMORYSTATUSEX);
-
-    if (! GlobalMemoryStatusEx(&memInfo) ) {
-        return PyErr_SetFromWindowsErr(0);
-    }
-
-    return Py_BuildValue("L", memInfo.ullTotalPageFile);
-}
-
-
-/*
- * Return a Python integer indicating the amount of available physical memory
- * in bytes.
- */
-static PyObject*
-get_avail_phymem(PyObject* self, PyObject* args)
-{
-    MEMORYSTATUSEX memInfo;
-    memInfo.dwLength = sizeof(MEMORYSTATUSEX);
-    if (! GlobalMemoryStatusEx(&memInfo) ) {
-        return PyErr_SetFromWindowsErr(0);
-    }
-    return Py_BuildValue("L", memInfo.ullAvailPhys);
-}
-
-
-/*
- * Return a Python integer indicating the amount of available virtual memory
- * in bytes.
- */
-static PyObject*
-get_avail_virtmem(PyObject* self, PyObject* args)
-{
-    MEMORYSTATUSEX memInfo;
-    memInfo.dwLength = sizeof(MEMORYSTATUSEX);
-
-    if (! GlobalMemoryStatusEx(&memInfo) ) {
-        return PyErr_SetFromWindowsErr(0);
-    }
-    return Py_BuildValue("L", memInfo.ullAvailPageFile);
+    return Py_BuildValue("(LLLLLLk)",
+                                memInfo.ullTotalPhys,      // total
+                                memInfo.ullAvailPhys,      // avail
+                                memInfo.ullTotalPageFile,  // total page file
+                                memInfo.ullAvailPageFile,  // avail page file
+                                memInfo.ullTotalVirtual,   // total virtual
+                                memInfo.ullAvailVirtual,   // avail virtual
+                                memInfo.dwMemoryLoad       // percent
+                                );
 }
 
 
@@ -537,164 +509,85 @@ get_avail_virtmem(PyObject* self, PyObject* args)
 
 
 /*
- * Return a Python tuple representing user, kernel and idle CPU times
+ * Return a Python list of tuples representing user, kernel and idle
+ * CPU times for every CPU on the system.
  */
 static PyObject*
 get_system_cpu_times(PyObject* self, PyObject* args)
 {
-    typedef BOOL (_stdcall *GST_PROC) (LPFILETIME, LPFILETIME, LPFILETIME);
-    static GST_PROC GetSystemTimes;
-    static BOOL bFirstCall = TRUE;
     float idle, kernel, user;
+    typedef DWORD (_stdcall *NTQSI_PROC) (int, PVOID, ULONG, PULONG);
+    NTQSI_PROC NtQuerySystemInformation;
+    HINSTANCE hNtDll;
+    SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION *sppi = NULL;
+    SYSTEM_INFO si;
+    UINT i;
+    PyObject *arg = NULL;
+    PyObject *retlist = PyList_New(0);
 
-    // Improves performance calling GetProcAddress only the first time
-    if (bFirstCall) {
-        // retrieves GetSystemTimes address in Kernel32
-        GetSystemTimes=(GST_PROC)GetProcAddress(GetModuleHandle
-                                               (TEXT("Kernel32.dll")),
-                                               "GetSystemTimes");
-        bFirstCall = FALSE;
-    }
+    // dynamic linking is mandatory to use NtQuerySystemInformation
+    hNtDll = LoadLibrary(TEXT("ntdll.dll"));
+    if (hNtDll != NULL) {
+        // gets NtQuerySystemInformation address
+        NtQuerySystemInformation = (NTQSI_PROC)GetProcAddress(
+                                    hNtDll, "NtQuerySystemInformation");
 
+        if (NtQuerySystemInformation != NULL)
+        {
+            // retrives number of processors
+            GetSystemInfo(&si);
 
-     // Uses GetSystemTimes if supported (winXP sp1+)
-    if (NULL!=GetSystemTimes) {
-        // GetSystemTimes supported
-
-        FILETIME idle_time;
-        FILETIME kernel_time;
-        FILETIME user_time;
-
-        if (!GetSystemTimes(&idle_time, &kernel_time, &user_time)) {
-            return PyErr_SetFromWindowsErr(0);
-        }
-
-        idle = (float)((HI_T * idle_time.dwHighDateTime) + \
-                       (LO_T * idle_time.dwLowDateTime));
-        user = (float)((HI_T * user_time.dwHighDateTime) + \
-                       (LO_T * user_time.dwLowDateTime));
-        kernel = (float)((HI_T * kernel_time.dwHighDateTime) + \
-                         (LO_T * kernel_time.dwLowDateTime));
-
-        // kernel time includes idle time on windows
-        // we return only busy kernel time subtracting idle time from kernel time
-        return Py_BuildValue("(fff)", user,
-                                      kernel - idle,
-                                      idle);
-
-    }
-
-    else {
-        // GetSystemTimes NOT supported, use NtQuerySystemInformation instead
-
-        typedef DWORD (_stdcall *NTQSI_PROC) (int, PVOID, ULONG, PULONG);
-        NTQSI_PROC NtQuerySystemInformation;
-        HINSTANCE hNtDll;
-        SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION *sppi = NULL;
-        SYSTEM_INFO si;
-        UINT i;
-
-        // dynamic linking is mandatory to use NtQuerySystemInformation
-        hNtDll = LoadLibrary(TEXT("ntdll.dll"));
-        if (hNtDll != NULL) {
-            // gets NtQuerySystemInformation address
-            NtQuerySystemInformation = (NTQSI_PROC)GetProcAddress(
-                                        hNtDll, "NtQuerySystemInformation");
-
-            if (NtQuerySystemInformation != NULL)
+            // allocates an array of SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION
+            // structures, one per processor
+            sppi = (SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION *) \
+                    malloc(si.dwNumberOfProcessors * \
+                         sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION));
+            if (sppi != NULL)
             {
-                // retrives number of processors
-                GetSystemInfo(&si);
-
-                // allocates an array of SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION
-                // structures, one per processor
-                sppi=(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION *) \
-                      malloc(si.dwNumberOfProcessors * \
-                             sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION));
-                if (sppi != NULL)
+                // gets cpu time informations
+                if (0 == NtQuerySystemInformation(
+                    SystemProcessorPerformanceInformation,
+                    sppi,
+                    si.dwNumberOfProcessors * sizeof
+                                (SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION),
+                    NULL)
+                )
                 {
-                    // gets cpu time informations
-                    if (0 == NtQuerySystemInformation(
-                                SystemProcessorPerformanceInformation,
-                                sppi,
-                                si.dwNumberOfProcessors * sizeof(SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION),
-                                NULL))
-                    {
-                        // computes system global times summing each processor value
-                        idle = user = kernel = 0;
-                        for (i=0; i<si.dwNumberOfProcessors; i++) {
-                            idle += (float)((HI_T * sppi[i].IdleTime.HighPart) + \
-                                            (LO_T * sppi[i].IdleTime.LowPart));
-                            user += (float)((HI_T * sppi[i].UserTime.HighPart) + \
-                                            (LO_T * sppi[i].UserTime.LowPart));
-                            kernel += (float)((HI_T * sppi[i].KernelTime.HighPart) + \
-                                              (LO_T * sppi[i].KernelTime.LowPart));
-                        }
-
+                    // computes system global times summing each processor value
+                    idle = user = kernel = 0;
+                    for (i=0; i<si.dwNumberOfProcessors; i++) {
+                        user = (float)((HI_T * sppi[i].UserTime.HighPart) + \
+                                       (LO_T * sppi[i].UserTime.LowPart));
+                        idle = (float)((HI_T * sppi[i].IdleTime.HighPart) + \
+                                       (LO_T * sppi[i].IdleTime.LowPart));
+                        kernel = (float)((HI_T * sppi[i].KernelTime.HighPart) + \
+                                         (LO_T * sppi[i].KernelTime.LowPart));
                         // kernel time includes idle time on windows
-                        // we return only busy kernel time subtracting idle
-                        // time from kernel time
-                        return Py_BuildValue("(ddd)", user,
-                                                      kernel - idle,
-                                                      idle
-                                             );
+                        // we return only busy kernel time subtracting
+                        // idle time from kernel time
+                        arg = Py_BuildValue("(ddd)", user,
+                                                     kernel - idle,
+                                                     idle);
+                        PyList_Append(retlist, arg);
+                        Py_XDECREF(arg);
+                    }
+                    free(sppi);
+                    FreeLibrary(hNtDll);
+                    return retlist;
 
-                    } // END NtQuerySystemInformation
+                }  // END NtQuerySystemInformation
+            }  // END malloc SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION
+        }  // END GetProcAddress
+    }  // END LoadLibrary
 
-                } // END malloc SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION
-
-            } // END GetProcAddress
-
-        } // END LoadLibrary
-
-        PyErr_SetFromWindowsErr(0);
-        if (sppi) {
-            free(sppi);
-        }
-        if (hNtDll) {
-            FreeLibrary(hNtDll);
-        }
-        return 0;
-
-    } // END GetSystemTimes NOT supported
-}
-
-
-/*
- * Sid to User convertion
- */
-BOOL SidToUser(PSID pSid, LPTSTR szUser, DWORD dwUserLen, LPTSTR szDomain, DWORD
-dwDomainLen, DWORD pid)
-{
-    SID_NAME_USE snuSIDNameUse;
-    DWORD dwULen;
-    DWORD dwDLen;
-
-    dwULen = dwUserLen;
-    dwDLen = dwDomainLen;
-
-    if ( IsValidSid( pSid ) ) {
-        // Get user and domain name based on SID
-        if ( LookupAccountSid( NULL, pSid, szUser, &dwULen, szDomain, &dwDLen, &snuSIDNameUse) ) {
-            // LocalSystem processes are incorrectly reported as owned
-            // by BUILTIN\Administrators We modify that behavior to
-            // conform to standard taskmanager only if the process is
-            // actually a System process
-            if (is_system_proc(pid) == 1) {
-                // default to *not* changing the data if we fail to
-                // check for local system privileges, so only look for
-                // definite confirmed system processes and ignore errors
-                if ( lstrcmpi(szDomain, TEXT("builtin")) == 0 && lstrcmpi(szUser, TEXT("administrators")) == 0) {
-                    strncpy  (szUser, "SYSTEM", dwUserLen);
-                    strncpy  (szDomain, "NT AUTHORITY", dwDomainLen);
-                }
-            }
-
-            return TRUE;
-        }
+    if (sppi) {
+        free(sppi);
     }
-
-    return FALSE;
+    if (hNtDll) {
+        FreeLibrary(hNtDll);
+    }
+    PyErr_SetFromWindowsErr(0);
+    return NULL;
 }
 
 
@@ -920,73 +813,26 @@ resume_process(PyObject* self, PyObject* args)
 static PyObject*
 get_process_num_threads(PyObject* self, PyObject* args)
 {
-    long pid;
-    int pid_return;
-    long nthreads = 0;
-    HANDLE hThreadSnap = NULL;
-    THREADENTRY32 te32 = {0};
+    DWORD pid;
+    PSYSTEM_PROCESS_INFORMATION process;
+    PVOID buffer;
+    int num;
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
-        return NULL;
-    if (pid == 0) {
-        // raise AD instead of returning 0 as procexp is able to
-        // retrieve useful information somehow
-        return AccessDenied();
-    }
-
-    pid_return = pid_is_running(pid);
-    if (pid_return == 0) {
-        return NoSuchProcess();
-    }
-    if (pid_return == -1) {
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
     }
-
-    hThreadSnap = CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD, 0);
-    if (hThreadSnap == INVALID_HANDLE_VALUE) {
-        PyErr_SetFromWindowsErr(0);
+    if (get_process_info(pid, &process, &buffer) != 1) {
+        free(buffer);
         return NULL;
     }
-
-    // Fill in the size of the structure before using it
-    te32.dwSize = sizeof(THREADENTRY32);
-
-    if (! Thread32First(hThreadSnap, &te32)) {
-        PyErr_SetFromWindowsErr(0);
-        CloseHandle(hThreadSnap);
-        return NULL;
-    }
-
-    // Walk the thread snapshot to find all threads of the process.
-    // If the thread belongs to the process, increase the counter.
-    do
-    {
-        if (te32.th32OwnerProcessID == pid)
-        {
-            HANDLE hThread = OpenThread(THREAD_QUERY_INFORMATION,
-                                        FALSE, te32.th32ThreadID);
-            if (hThread == NULL) {
-                if (GetLastError() == ERROR_INVALID_PARAMETER) {
-                    NoSuchProcess();
-                }
-                else {
-                    PyErr_SetFromWindowsErr(0);
-                }
-                CloseHandle(hThread);
-                CloseHandle(hThreadSnap);
-                return NULL;
-            }
-            nthreads += 1;
-            CloseHandle(hThread);
-        }
-    } while (Thread32Next(hThreadSnap, &te32));
-
-    // every process should be supposed to have at least one thread
-    if (nthreads == 0) {
+    if (pid_is_running(pid) == 0) {
+        free(buffer);
         return NoSuchProcess();
     }
 
-    return Py_BuildValue("l", nthreads);
+    num = (int)process->NumberOfThreads;
+    free(buffer);
+    return Py_BuildValue("i", num);
 }
 
 
@@ -1002,8 +848,9 @@ get_process_threads(PyObject* self, PyObject* args)
     int rc;
     FILETIME ftDummy, ftKernel, ftUser;
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
+    }
     if (pid == 0) {
         // raise AD instead of returning 0 as procexp is able to
         // retrieve useful information somehow
@@ -1042,21 +889,15 @@ get_process_threads(PyObject* self, PyObject* args)
             HANDLE hThread = OpenThread(THREAD_QUERY_INFORMATION,
                                         FALSE, te32.th32ThreadID);
             if (hThread == NULL) {
-                if (GetLastError() == ERROR_INVALID_PARAMETER) {
-                    NoSuchProcess();
-                }
-                else {
-                    PyErr_SetFromWindowsErr(0);
-                }
-                CloseHandle(hThread);
-                CloseHandle(hThreadSnap);
-                return NULL;
+                // thread has disappeared on us
+                continue;
             }
 
             rc = GetThreadTimes(hThread, &ftDummy, &ftDummy, &ftKernel, &ftUser);
             if (rc == 0) {
                 PyErr_SetFromWindowsErr(0);
                 CloseHandle(hThread);
+                CloseHandle(hThreadSnap);
                 return NULL;
             }
 
@@ -1084,6 +925,7 @@ get_process_threads(PyObject* self, PyObject* args)
         }
     } while (Thread32Next(hThreadSnap, &te32));
 
+    CloseHandle(hThreadSnap);
     return retList;
 }
 
@@ -1107,6 +949,7 @@ get_process_open_files(PyObject* self, PyObject* args)
     }
 
     filesList = get_open_files(pid, processHandle);
+    CloseHandle(processHandle);
     if (filesList == NULL) {
         return PyErr_SetFromWindowsErr(0);
     }
@@ -1120,7 +963,7 @@ get_process_open_files(PyObject* self, PyObject* args)
  If no match is found return an empty string.
 */
 static PyObject*
-_QueryDosDevice(PyObject* self, PyObject* args)
+win32_QueryDosDevice(PyObject* self, PyObject* args)
 {
     LPCTSTR   lpDevicePath;
     TCHAR d = TEXT('A');
@@ -1445,8 +1288,9 @@ get_process_connections(PyObject* self, PyObject* args)
 
         for (i = 0; i < tcp4Table->dwNumEntries; i++)
         {
-            if (tcp4Table->table[i].dwOwningPid != pid)
+            if (tcp4Table->table[i].dwOwningPid != pid) {
                 continue;
+            }
 
             if (tcp4Table->table[i].dwLocalAddr != 0 ||
                 tcp4Table->table[i].dwLocalPort != 0)
@@ -1576,8 +1420,9 @@ get_process_connections(PyObject* self, PyObject* args)
 
         for (i = 0; i < udp4Table->dwNumEntries; i++)
         {
-            if (udp4Table->table[i].dwOwningPid != pid)
+            if (udp4Table->table[i].dwOwningPid != pid) {
                 continue;
+            }
 
             if (udp4Table->table[i].dwLocalAddr != 0 ||
                 udp4Table->table[i].dwLocalPort != 0)
@@ -1677,6 +1522,7 @@ get_process_priority(PyObject* self, PyObject* args)
     }
 
     priority = GetPriorityClass(hProcess);
+    CloseHandle(hProcess);
     if (priority == 0) {
         PyErr_SetFromWindowsErr(0);
         return NULL;
@@ -1706,6 +1552,7 @@ set_process_priority(PyObject* self, PyObject* args)
     }
 
     retval = SetPriorityClass(hProcess, priority);
+    CloseHandle(hProcess);
     if (retval == 0) {
         PyErr_SetFromWindowsErr(0);
         return NULL;
@@ -1736,8 +1583,10 @@ get_process_io_counters(PyObject* self, PyObject* args)
         return NULL;
     }
     if (! GetProcessIoCounters(hProcess, &IoCounters)) {
+        CloseHandle(hProcess);
         return PyErr_SetFromWindowsErr(0);
     }
+    CloseHandle(hProcess);
     return Py_BuildValue("(KKKK)", IoCounters.ReadOperationCount,
                                    IoCounters.WriteOperationCount,
                                    IoCounters.ReadTransferCount,
@@ -1755,24 +1604,130 @@ is_process_suspended(PyObject* self, PyObject* args)
     DWORD pid;
     ULONG i;
     PSYSTEM_PROCESS_INFORMATION process;
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    PVOID buffer;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
         return NULL;
-    if (get_process_info(pid, &process) != 1)
+    }
+    if (get_process_info(pid, &process, &buffer) != 1) {
+        free(buffer);
         return NULL;
-    if (pid_is_running(pid) == 0)
+    }
+    if (pid_is_running(pid) == 0) {
+        free(buffer);
         return NoSuchProcess();
+    }
 
     for (i = 0; i < process->NumberOfThreads; i++) {
         if (process->Threads[i].ThreadState != Waiting ||
             process->Threads[i].WaitReason != Suspended)
         {
+            free(buffer);
             Py_RETURN_FALSE;
         }
     }
+    free(buffer);
     Py_RETURN_TRUE;
 }
 
 
+/*
+ * Return path's disk total and free as a Python tuple.
+ */
+static PyObject*
+get_disk_usage(PyObject* self, PyObject* args)
+{
+    BOOL retval;
+    ULARGE_INTEGER _, total, free;
+    LPCTSTR path;
+
+    if (! PyArg_ParseTuple(args, "s", &path)) {
+        return NULL;
+    }
+
+    Py_BEGIN_ALLOW_THREADS
+    retval = GetDiskFreeSpaceEx(path, &_, &total, &free);
+    Py_END_ALLOW_THREADS
+    if (retval == 0) {
+        return PyErr_SetFromWindowsErr(0);
+    }
+
+    return Py_BuildValue("(LL)", total.QuadPart, free.QuadPart);
+}
+
+
+/*
+ * Return disk partitions as a list of namedtuples.
+ */
+static PyObject*
+win32_GetLogicalDriveStrings(PyObject* self, PyObject* args)
+{
+    DWORD num_bytes;
+    char drive_strings[255];
+    char* drive_letter = drive_strings;
+    PyObject* py_retlist = PyList_New(0);
+
+    Py_BEGIN_ALLOW_THREADS
+    num_bytes = GetLogicalDriveStrings(254, drive_letter);
+    Py_END_ALLOW_THREADS
+
+    if (num_bytes == 0) {
+        return PyErr_SetFromWindowsErr(0);
+    }
+
+    while (*drive_letter != 0) {
+        PyList_Append(py_retlist, Py_BuildValue("s", drive_letter));
+        drive_letter = strchr(drive_letter, 0) +1;
+    }
+
+    return py_retlist;
+}
+
+
+static PyObject*
+win32_GetDriveType(PyObject* self, PyObject* args)
+{
+    LPCTSTR drive_letter;
+    int type;
+    char* type_str;
+
+    if (! PyArg_ParseTuple(args, "s", &drive_letter)) {
+        return NULL;
+    }
+
+    Py_BEGIN_ALLOW_THREADS
+    type = GetDriveType(drive_letter);
+    Py_END_ALLOW_THREADS
+
+    switch (type) {
+        case DRIVE_UNKNOWN:
+            type_str = "unknown";
+            break;
+        case DRIVE_NO_ROOT_DIR:
+            type_str = "unmounted";
+        case DRIVE_REMOVABLE:
+            type_str = "removable";
+            break;
+        case DRIVE_FIXED:
+            type_str = "fixed";
+            break;
+        case DRIVE_REMOTE:
+            type_str = "remote";
+            break;
+        case DRIVE_CDROM:
+            type_str = "cdrom";
+            break;
+        case DRIVE_RAMDISK:
+            type_str = "ramdisk";
+            break;
+        default:
+            type_str = "?";
+            break;
+    }
+
+    return Py_BuildValue("s", type_str);
+}
+
 
 // ------------------------ Python init ---------------------------
 
@@ -1804,8 +1759,6 @@ PsutilMethods[] =
         "Resume a process"},
     {"get_process_open_files", get_process_open_files, METH_VARARGS,
         "Return files opened by process"},
-    {"_QueryDosDevice", _QueryDosDevice, METH_VARARGS,
-        "QueryDosDevice binding"},
     {"get_process_username", get_process_username, METH_VARARGS,
         "Return the username of a process"},
     {"get_process_connections", get_process_connections, METH_VARARGS,
@@ -1825,7 +1778,6 @@ PsutilMethods[] =
     {"is_process_suspended", is_process_suspended, METH_VARARGS,
         "Return True if one of the process threads is in a suspended state"},
 
-
     // --- system-related functions
 
      {"get_pid_list", get_pid_list, METH_VARARGS,
@@ -1836,16 +1788,20 @@ PsutilMethods[] =
          "Returns the number of CPUs on the system"},
      {"get_system_uptime", get_system_uptime, METH_VARARGS,
          "Return system uptime"},
-     {"get_total_phymem", get_total_phymem, METH_VARARGS,
+     {"get_system_phymem", get_system_phymem, METH_VARARGS,
          "Return the total amount of physical memory, in bytes"},
-     {"get_total_virtmem", get_total_virtmem, METH_VARARGS,
-         "Return the total amount of virtual memory, in bytes"},
-     {"get_avail_phymem", get_avail_phymem, METH_VARARGS,
-         "Return the amount of available physical memory, in bytes"},
-     {"get_avail_virtmem", get_avail_virtmem, METH_VARARGS,
-         "Return the amount of available virtual memory, in bytes"},
      {"get_system_cpu_times", get_system_cpu_times, METH_VARARGS,
-         "Return system cpu times as a tuple (user, system, idle)"},
+         "Return system per-cpu times as a list of tuples"},
+     {"get_disk_usage", get_disk_usage, METH_VARARGS,
+         "Return path's disk total and free as a Python tuple."},
+
+     // --- windows API bindings
+     {"win32_GetLogicalDriveStrings", win32_GetLogicalDriveStrings, METH_VARARGS,
+         "GetLogicalDriveStrings binding"},
+     {"win32_GetDriveType", win32_GetDriveType, METH_VARARGS,
+         "GetDriveType binding"},
+     {"win32_QueryDosDevice", win32_QueryDosDevice, METH_VARARGS,
+         "QueryDosDevice binding"},
 
      {NULL, NULL, 0, NULL}
 };
