@@ -159,6 +159,24 @@ get_process_gids(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return process controlling terminal number as an integer.
+ */
+static PyObject*
+get_process_tty_nr(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    return Py_BuildValue("i", kp.kp_eproc.e_tdev);
+}
+
+
 /*
  * Return a Python integer indicating the number of CPUs on the system.
  */
@@ -351,7 +369,7 @@ static PyObject*
 get_process_num_threads(PyObject* self, PyObject* args)
 {
     long pid;
-    int err;
+    int err, ret;
     unsigned int info_count = TASK_BASIC_INFO_COUNT;
     mach_port_t task;
     struct task_basic_info tasks_info;
@@ -382,14 +400,19 @@ get_process_num_threads(PyObject* self, PyObject* args)
                 return PyErr_Format(PyExc_RuntimeError,
                                     "task_info(TASK_BASIC_INFO) failed");
         }
-
         err = task_threads(task, &thread_list, &thread_count);
         if (err == KERN_SUCCESS) {
+            ret = vm_deallocate(task, (vm_address_t)thread_list,
+                                thread_count * sizeof(int));
+            if (ret != KERN_SUCCESS) {
+                printf("vm_deallocate() failed\n");
+            }
             return Py_BuildValue("l", (long)thread_count);
         }
+        else {
+            return PyErr_Format(PyExc_RuntimeError, "task_thread() failed");
+        }
     }
-
-
     else {
         if (! pid_exists(pid) ) {
             return NoSuchProcess();
@@ -523,6 +546,50 @@ get_system_cpu_times(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return a Python list of tuple representing per-cpu times
+ */
+static PyObject*
+get_system_per_cpu_times(PyObject* self, PyObject* args)
+{
+    natural_t cpu_count;
+    processor_info_array_t info_array;
+    mach_msg_type_number_t info_count;
+    kern_return_t error;
+    processor_cpu_load_info_data_t* cpu_load_info;
+    PyObject* py_retlist = PyList_New(0);
+    PyObject* py_cputime;
+    int i, ret;
+
+    error = host_processor_info(mach_host_self(), PROCESSOR_CPU_LOAD_INFO,
+                                &cpu_count, &info_array, &info_count);
+    if (error != KERN_SUCCESS) {
+        return PyErr_Format(PyExc_RuntimeError,
+              "Error in host_processor_info(): %s", mach_error_string(error));
+    }
+
+    cpu_load_info = (processor_cpu_load_info_data_t*) info_array;
+
+    for (i = 0; i < cpu_count; i++) {
+        py_cputime = Py_BuildValue("(dddd)",
+               (double)cpu_load_info[i].cpu_ticks[CPU_STATE_USER] / CLK_TCK,
+               (double)cpu_load_info[i].cpu_ticks[CPU_STATE_NICE] / CLK_TCK,
+               (double)cpu_load_info[i].cpu_ticks[CPU_STATE_SYSTEM] / CLK_TCK,
+               (double)cpu_load_info[i].cpu_ticks[CPU_STATE_IDLE] / CLK_TCK
+              );
+        PyList_Append(py_retlist, py_cputime);
+        Py_XDECREF(py_cputime);
+    }
+
+    ret = vm_deallocate(mach_task_self(), (vm_address_t)info_array,
+                        info_count * sizeof(int));
+    if (ret != KERN_SUCCESS) {
+        printf("vm_deallocate() failed\n");
+    }
+    return py_retlist;
+}
+
+
 /*
  * Return a Python float indicating the system boot time expressed in
  * seconds since the epoch.
@@ -545,6 +612,54 @@ get_system_boot_time(PyObject* self, PyObject* args)
 }
 
 
+/*
+ * Return a list of tuples including device, mount point and fs type
+ * for all partitions mounted on the system.
+ */
+static PyObject*
+get_disk_partitions(PyObject* self, PyObject* args)
+{
+    int num;
+    int i;
+    long len;
+    struct statfs *fs;
+    PyObject* py_retlist = PyList_New(0);
+    PyObject* py_tuple;
+
+    // get the number of mount points
+    Py_BEGIN_ALLOW_THREADS
+    num = getfsstat(NULL, 0, MNT_NOWAIT);
+    Py_END_ALLOW_THREADS
+    if (num == -1) {
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    len = sizeof(*fs) * num;
+    fs = malloc(len);
+
+    Py_BEGIN_ALLOW_THREADS
+    num = getfsstat(fs, len, MNT_NOWAIT);
+    Py_END_ALLOW_THREADS
+    if (num == -1) {
+        free(fs);
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    for (i = 0; i < num; i++) {
+        py_tuple = Py_BuildValue("(sss)", fs[i].f_mntfromname,  // device
+                                          fs[i].f_mntonname,    // mount point
+                                          fs[i].f_fstypename);  // fs type
+        PyList_Append(py_retlist, py_tuple);
+        Py_XDECREF(py_tuple);
+    }
+
+    free(fs);
+    return py_retlist;
+}
+
+
 /*
  * Return process status as a Python integer.
  */
@@ -570,7 +685,7 @@ static PyObject*
 get_process_threads(PyObject* self, PyObject* args)
 {
     long pid;
-    int err, j;
+    int err, j, ret;
     kern_return_t kr;
     unsigned int info_count = TASK_BASIC_INFO_COUNT;
     mach_port_t task;
@@ -620,7 +735,7 @@ get_process_threads(PyObject* self, PyObject* args)
                          (thread_info_t)thinfo, &thread_info_count);
         if (kr != KERN_SUCCESS) {
             return PyErr_Format(PyExc_RuntimeError, "thread_info() failed");
-        }
+        }        
         basic_info_th = (thread_basic_info_t)thinfo;
         // XXX - thread_info structure does not provide any process id;
         // the best we can do is assigning an incremental bogus value
@@ -632,6 +747,12 @@ get_process_threads(PyObject* self, PyObject* args)
         Py_XDECREF(pyTuple);
     }
 
+    ret = vm_deallocate(task, (vm_address_t)thread_list,
+                        thread_count * sizeof(int));
+    if (ret != KERN_SUCCESS) {
+        printf("vm_deallocate() failed\n");
+    }
+
     return retList;
 }
 
@@ -796,6 +917,10 @@ get_process_connections(PyObject* self, PyObject* args)
         return NULL;
     }
 
+    if (pid == 0) {
+        return retList;
+    }
+
     pidinfo_result = proc_pidinfo(pid, PROC_PIDLISTFDS, 0, NULL, 0);
     if (pidinfo_result <= 0) {
         goto error;
@@ -962,7 +1087,8 @@ PsutilMethods[] =
          "Return files opened by process as a list of tuples"},
      {"get_process_connections", get_process_connections, METH_VARARGS,
          "Get process TCP and UDP connections as a list of tuples"},
-
+     {"get_process_tty_nr", get_process_tty_nr, METH_VARARGS,
+         "Return process tty number as an integer"},
 
      // --- system-related functions
 
@@ -980,9 +1106,14 @@ PsutilMethods[] =
          "Return the amount of available virtual memory, in bytes"},
      {"get_system_cpu_times", get_system_cpu_times, METH_VARARGS,
          "Return system cpu times as a tuple (user, system, nice, idle, irc)"},
+     {"get_system_per_cpu_times", get_system_per_cpu_times, METH_VARARGS,
+         "Return system per-cpu times as a list of tuples"},
      {"get_system_boot_time", get_system_boot_time, METH_VARARGS,
          "Return a float indicating the system boot time expressed in "
          "seconds since the epoch"},
+     {"get_disk_partitions", get_disk_partitions, METH_VARARGS,
+         "Return a list of tuples including device, mount point and "
+         "fs type for all partitions mounted on the system."},
 
      {NULL, NULL, 0, NULL}
 };
@@ -996,7 +1127,6 @@ struct module_state {
 #define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
 #else
 #define GETSTATE(m) (&_state)
-static struct module_state _state;
 #endif
 
 #if PY_MAJOR_VERSION >= 3
@@ -1055,13 +1185,6 @@ init_psutil_osx(void)
     if (module == NULL) {
         INITERROR;
     }
-    struct module_state *st = GETSTATE(module);
-
-    st->error = PyErr_NewException("_psutil_osx.Error", NULL, NULL);
-    if (st->error == NULL) {
-        Py_DECREF(module);
-        INITERROR;
-    }
 #if PY_MAJOR_VERSION >= 3
     return module;
 #endif
