@@ -142,39 +142,6 @@ get_pids(DWORD *numberOfReturnedPIDs) {
 }
 
 
-int
-is_system_proc(DWORD pid) {
-    HANDLE hProcess;
-
-    // Special case for PID 0 System Idle Process
-    // and PID 4 (SYSTEM)
-    if ((pid == 0) || (pid == 4)) {
-        return 1;
-    }
-    if (pid < 0) {
-        return 0;
-    }
-
-    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, pid);
-    if (NULL == hProcess) {
-        // invalid parameter is no such process
-        if (GetLastError() == ERROR_INVALID_PARAMETER) {
-            return 0;
-        }
-
-        // access denied obviously means there's a process to deny access to...
-        if (GetLastError() == ERROR_ACCESS_DENIED) {
-            return 1;
-        }
-
-        PyErr_SetFromWindowsErr(0);
-        return -1;
-    }
-
-    return HasSystemPrivilege(hProcess);
-}
-
-
 int
 pid_is_running(DWORD pid)
 {
@@ -306,7 +273,7 @@ get_ppid(long pid)
     if( Process32First(h, &pe)) {
         do {
             if (pe.th32ProcessID == pid) {
-                //printf("PID: %i; PPID: %i\n", pid, pe.th32ParentProcessID);
+                ////printf("PID: %i; PPID: %i\n", pid, pe.th32ParentProcessID);
                 CloseHandle(h);
                 return Py_BuildValue("I", pe.th32ParentProcessID);
             }
@@ -342,7 +309,6 @@ get_arg_list(long pid)
     PyObject *arg_from_wchar = NULL;
     PyObject *argList = NULL;
 
-
     hProcess = handle_from_pid(pid);
     if(hProcess == NULL) {
         return NULL;
@@ -359,7 +325,7 @@ get_arg_list(long pid)
         &rtlUserProcParamsAddress, sizeof(PVOID), NULL))
 #endif
     {
-        //printf("Could not read the address of ProcessParameters!\n");
+        ////printf("Could not read the address of ProcessParameters!\n");
         PyErr_SetFromWindowsErr(0);
         CloseHandle(hProcess);
         return NULL;
@@ -374,9 +340,9 @@ get_arg_list(long pid)
         &commandLine, sizeof(commandLine), NULL))
 #endif
     {
-        //printf("Could not read CommandLine!\n");
-        PyErr_SetFromWindowsErr(0);
+        ////printf("Could not read CommandLine!\n");
         CloseHandle(hProcess);
+        PyErr_SetFromWindowsErr(0);
         return NULL;
     }
 
@@ -388,15 +354,15 @@ get_arg_list(long pid)
     if (!ReadProcessMemory(hProcess, commandLine.Buffer,
         commandLineContents, commandLine.Length, NULL))
     {
-        //printf("Could not read the command line string!\n");
-        PyErr_SetFromWindowsErr(0);
+        ////printf("Could not read the command line string!\n");
         CloseHandle(hProcess);
+        PyErr_SetFromWindowsErr(0);
         free(commandLineContents);
         return NULL;
     }
 
     /* print the commandline */
-    //printf("%.*S\n", commandLine.Length / 2, commandLineContents);
+    ////printf("%.*S\n", commandLine.Length / 2, commandLineContents);
 
     // null-terminate the string to prevent wcslen from returning incorrect length
     // the length specifier is in characters, but commandLine.Length is in bytes
@@ -421,7 +387,7 @@ get_arg_list(long pid)
         // string object and add to arg list
         argList = Py_BuildValue("[]");
         for(i=0; i<nArgs; i++) {
-            //printf("%d: %.*S (%d characters)\n", i, wcslen(szArglist[i]),
+            ////printf("%d: %.*S (%d characters)\n", i, wcslen(szArglist[i]),
             //                  szArglist[i], wcslen(szArglist[i]));
             arg_from_wchar = PyUnicode_FromWideChar(szArglist[i],
                                                     wcslen(szArglist[i])
@@ -462,7 +428,7 @@ const STATUS_BUFFER_TOO_SMALL = 0xC0000023L;
  * On success return 1, else 0 with Python exception already set.
  */
 int
-get_process_info(DWORD pid, PSYSTEM_PROCESS_INFORMATION *retProcess)
+get_process_info(DWORD pid, PSYSTEM_PROCESS_INFORMATION *retProcess, PVOID *retBuffer)
 {
     static ULONG initialBufferSize = 0x4000;
     NTSTATUS status;
@@ -496,25 +462,23 @@ get_process_info(DWORD pid, PSYSTEM_PROCESS_INFORMATION *retProcess)
     }
 
     if (status != 0) {
-        free(buffer);
         PyErr_Format(PyExc_RuntimeError, "NtQuerySystemInformation() failed");
         return 0;
     }
 
-
-    if (bufferSize <= 0x20000)
+    if (bufferSize <= 0x20000) {
         initialBufferSize = bufferSize;
+    }
 
     process = PH_FIRST_PROCESS(buffer);
     do {
         if (process->UniqueProcessId == (HANDLE)pid) {
-            free(buffer);
             *retProcess = process;
+            *retBuffer = buffer;
             return 1;
         }
-    } while (process = PH_NEXT_PROCESS(process));
+    } while ( (process = PH_NEXT_PROCESS(process)) );
 
-    free(buffer);
     NoSuchProcess();
     return 0;
 }
