@@ -8,8 +8,6 @@
 #include <Python.h>
 #include <windows.h>
 
-PyObject * NoSuchProcess(void);
-PyObject * AccessDenied(void);
 HANDLE handle_from_pid_waccess(DWORD pid, DWORD dwDesiredAccess);
 HANDLE handle_from_pid(DWORD pid);
 PVOID GetPebAddress(HANDLE ProcessHandle);
@@ -17,9 +15,7 @@ HANDLE handle_from_pid(DWORD pid);
 BOOL is_running(HANDLE hProcess);
 int pid_in_proclist(DWORD pid);
 int pid_is_running(DWORD pid);
-int is_system_proc(DWORD pid);
 PyObject* get_arg_list(long pid);
 PyObject* get_ppid(long pid);
 PyObject* get_name(long pid);
 DWORD* get_pids(DWORD *numberOfReturnedPIDs);
-
