@@ -6,12 +6,11 @@
 import unittest
 import subprocess
 import time
-import re
 import sys
 
 import psutil
 
-from test_psutil import reap_children, get_test_subprocess
+from test_psutil import reap_children, get_test_subprocess, sh
 #from _posix import ps
 
 
@@ -53,6 +52,33 @@ class OSXSpecificTestCase(unittest.TestCase):
                                      time.localtime(start_psutil))
         self.assertEqual(start_ps, start_psutil)
 
+    def test_disks(self):
+        # test psutil.disk_usage() and psutil.disk_partitions()
+        # against "df -a"
+        def df(path):
+            out = sh('df -k "%s"' % path).strip()
+            lines = out.split('\n')
+            lines.pop(0)
+            line = lines.pop(0)
+            dev, total, used, free = line.split()[:4]
+            if dev == 'none':
+                dev = ''
+            total = int(total) * 1024
+            used = int(used) * 1024
+            free = int(free) * 1024
+            return dev, total, used, free
+
+        for part in psutil.disk_partitions(all=False):
+            usage = psutil.disk_usage(part.mountpoint)
+            dev, total, used, free = df(part.mountpoint)
+            self.assertEqual(part.device, dev)
+            self.assertEqual(usage.total, total)
+            # 10 MB tollerance
+            if abs(usage.free - free) > 10 * 1024 * 1024:
+                self.fail("psutil=%s, df=%s" % usage.free, free)
+            if abs(usage.used - used) > 10 * 1024 * 1024:
+                self.fail("psutil=%s, df=%s" % usage.used, used)
+
 
 if __name__ == '__main__':
     test_suite = unittest.TestSuite()
