@@ -12,7 +12,7 @@ import os
 import psutil
 
 from test_psutil import (get_test_subprocess, reap_children, PYTHON, LINUX, OSX,
-                         ignore_access_denied)
+                         ignore_access_denied, sh)
 
 
 def ps(cmd):
@@ -88,10 +88,7 @@ class PosixSpecificTestCase(unittest.TestCase):
         # remove path if there is any, from the command
         name_ps = os.path.basename(name_ps)
         name_psutil = psutil.Process(self.pid).name
-        if OSX:
-            self.assertEqual(name_psutil, "Python")
-        else:
-            self.assertEqual(name_ps, name_psutil)
+        self.assertEqual(name_ps, name_psutil)
 
     def test_process_exe(self):
         ps_pathname = ps("ps --no-headers -o command -p %s" %self.pid).split(' ')[0]
@@ -105,9 +102,8 @@ class PosixSpecificTestCase(unittest.TestCase):
             # "/usr/local/bin/python"
             # We do not want to consider this difference in accuracy
             # an error.
-            ps_extended_pathname = PYTHON + "%s.%s" % (sys.version_info.major,
-                                                       sys.version_info.minor)
-            self.assertEqual(ps_extended_pathname, psutil_pathname)
+            adjusted_ps_pathname = ps_pathname[:len(ps_pathname)]
+            self.assertEqual(ps_pathname, adjusted_ps_pathname)
 
     def test_process_cmdline(self):
         ps_cmdline = ps("ps --no-headers -o command -p %s" %self.pid)
@@ -139,8 +135,8 @@ class PosixSpecificTestCase(unittest.TestCase):
         pids_psutil.sort()
 
         if pids_ps != pids_psutil:
-            difference = filter(lambda x:x not in pids_ps, pids_psutil) + \
-                         filter(lambda x:x not in pids_psutil, pids_ps)
+            difference = [x for x in pids_psutil if x not in pids_ps] + \
+                         [x for x in pids_ps if x not in pids_psutil]
             self.fail("difference: " + str(difference))
 
 
